@@ -1,5 +1,7 @@
 #include <xml_node.h>
 
+static ID decorate, decorate_bang;
+
 #ifdef DEBUG
 static void debug_node_dealloc(xmlNodePtr x)
 {
@@ -12,7 +14,10 @@ static void debug_node_dealloc(xmlNodePtr x)
 
 static void mark(xmlNodePtr node)
 {
-  rb_gc_mark(DOC_RUBY_OBJECT(node->doc));
+  // it's OK if the document isn't fully realized (as in XML::Reader).
+  // see http://github.com/tenderlove/nokogiri/issues/closed/#issue/95
+  if (DOC_RUBY_OBJECT_TEST(node->doc) && DOC_RUBY_OBJECT(node->doc))
+    rb_gc_mark(DOC_RUBY_OBJECT(node->doc));
 }
 
 /* :nodoc: */
@@ -21,8 +26,11 @@ typedef xmlNodePtr (*node_other_func)(xmlNodePtr, xmlNodePtr);
 /* :nodoc: */
 static void relink_namespace(xmlNodePtr reparented)
 {
+  // Avoid segv when relinking against unlinked nodes.
+  if(!reparented->parent) return;
+
   // Make sure that our reparented node has the correct namespaces
-  if(reparented->doc != (xmlDocPtr)reparented->parent)
+  if(!reparented->ns && reparented->doc != (xmlDocPtr)reparented->parent)
     xmlSetNs(reparented, reparented->parent->ns);
 
   // Search our parents for an existing definition
@@ -54,24 +62,47 @@ static VALUE reparent_node_with(VALUE node_obj, VALUE other_obj, node_other_func
   VALUE reparented_obj ;
   xmlNodePtr node, other, reparented ;
 
-  if(! rb_funcall(node_obj, rb_intern("is_a?"), 1, cNokogiriXmlNode))
+  if(!rb_obj_is_kind_of(node_obj, cNokogiriXmlNode))
     rb_raise(rb_eArgError, "node must be a Nokogiri::XML::Node");
 
   Data_Get_Struct(node_obj, xmlNode, node);
   Data_Get_Struct(other_obj, xmlNode, other);
 
+  if(XML_DOCUMENT_NODE == node->type || XML_HTML_DOCUMENT_NODE == node->type)
+    rb_raise(rb_eArgError, "cannot reparent a document node");
+
+  // If a document fragment is added, we need to reparent all of it's children
+  if(node->type == XML_DOCUMENT_FRAG_NODE)
+  {
+    xmlNodePtr child = node->children;
+    while(NULL != child) {
+      reparent_node_with(Nokogiri_wrap_xml_node((VALUE)NULL, child), other_obj, func);
+      child = child->next;
+    }
+    return node_obj;
+  }
+
+  if(node->type == XML_TEXT_NODE) {
+    NOKOGIRI_ROOT_NODE(node);
+    node = xmlDocCopyNode(node, other->doc, 1);
+  }
+
   if (node->doc == other->doc) {
     xmlUnlinkNode(node) ;
+
+    // TODO: I really want to remove this.  We shouldn't support 2.6.16 anymore
     if ( node->type == XML_TEXT_NODE
          && other->type == XML_TEXT_NODE
          && is_2_6_16() ) {
-      other->content = xmlStrdup(other->content); // we'd rather leak than segfault.
+
+      // we'd rather leak than segfault.
+      other->content = xmlStrdup(other->content);
+
     }
 
     if(!(reparented = (*func)(other, node))) {
       rb_raise(rb_eRuntimeError, "Could not reparent node (1)");
     }
-
   } else {
     xmlNodePtr duped_node ;
     // recursively copy to the new document
@@ -96,7 +127,7 @@ static VALUE reparent_node_with(VALUE node_obj, VALUE other_obj, node_other_func
 
   reparented_obj = Nokogiri_wrap_xml_node(Qnil, reparented);
 
-  rb_funcall(reparented_obj, rb_intern("decorate!"), 0);
+  rb_funcall(reparented_obj, decorate_bang, 0);
 
   return reparented_obj ;
 }
@@ -126,7 +157,7 @@ static VALUE pointer_id(VALUE self)
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, node);
 
-  return INT2NUM((int)(node));
+  return INT2NUM((long)(node));
 }
 
 /*
@@ -144,12 +175,92 @@ static VALUE encode_special_chars(VALUE self, VALUE string)
       (const xmlChar *)StringValuePtr(string)
   );
 
-  VALUE encoded_str = NOKOGIRI_STR_NEW2(encoded, node->doc->encoding);
+  VALUE encoded_str = NOKOGIRI_STR_NEW2(encoded);
   xmlFree(encoded);
 
   return encoded_str;
 }
 
+/*
+ * call-seq:
+ *  create_internal_subset(name, external_id, system_id)
+ *
+ * Create an internal subset
+ */
+static VALUE create_internal_subset(VALUE self, VALUE name, VALUE external_id, VALUE system_id)
+{
+  xmlNodePtr node;
+  xmlDocPtr doc;
+  Data_Get_Struct(self, xmlNode, node);
+
+  doc = node->doc;
+
+  if(xmlGetIntSubset(doc))
+    rb_raise(rb_eRuntimeError, "Document already has an internal subset");
+
+  xmlDtdPtr dtd = xmlCreateIntSubset(
+      doc, 
+      NIL_P(name)        ? NULL : (const xmlChar *)StringValuePtr(name),
+      NIL_P(external_id) ? NULL : (const xmlChar *)StringValuePtr(external_id),
+      NIL_P(system_id)   ? NULL : (const xmlChar *)StringValuePtr(system_id)
+  );
+
+  if(!dtd) return Qnil;
+
+  return Nokogiri_wrap_xml_node(Qnil, (xmlNodePtr)dtd);
+}
+
+/*
+ * call-seq:
+ *  create_external_subset(name, external_id, system_id)
+ *
+ * Create an external subset
+ */
+static VALUE create_external_subset(VALUE self, VALUE name, VALUE external_id, VALUE system_id)
+{
+  xmlNodePtr node;
+  xmlDocPtr doc;
+  Data_Get_Struct(self, xmlNode, node);
+
+  doc = node->doc;
+
+  if(doc->extSubset)
+    rb_raise(rb_eRuntimeError, "Document already has an external subset");
+
+  xmlDtdPtr dtd = xmlNewDtd(
+      doc, 
+      NIL_P(name)        ? NULL : (const xmlChar *)StringValuePtr(name),
+      NIL_P(external_id) ? NULL : (const xmlChar *)StringValuePtr(external_id),
+      NIL_P(system_id)   ? NULL : (const xmlChar *)StringValuePtr(system_id)
+  );
+
+  if(!dtd) return Qnil;
+
+  return Nokogiri_wrap_xml_node(Qnil, (xmlNodePtr)dtd);
+}
+
+/*
+ * call-seq:
+ *  external_subset
+ *
+ * Get the external subset
+ */
+static VALUE external_subset(VALUE self)
+{
+  xmlNodePtr node;
+  xmlDocPtr doc;
+  Data_Get_Struct(self, xmlNode, node);
+
+  if(!node->doc) return Qnil;
+
+  doc = node->doc;
+  xmlDtdPtr dtd = doc->extSubset;
+
+  if(!dtd) return Qnil;
+
+  return Nokogiri_wrap_xml_node(Qnil, (xmlNodePtr)dtd);
+}
+
 /*
  * call-seq:
  *  internal_subset
@@ -184,12 +295,12 @@ static VALUE duplicate_node(int argc, VALUE *argv, VALUE self)
   VALUE level;
 
   if(rb_scan_args(argc, argv, "01", &level) == 0)
-    level = INT2NUM(1);
+    level = INT2NUM((long)1);
 
   xmlNodePtr node, dup;
   Data_Get_Struct(self, xmlNode, node);
 
-  dup = xmlDocCopyNode(node, node->doc, NUM2INT(level));
+  dup = xmlDocCopyNode(node, node->doc, (int)NUM2INT(level));
   if(dup == NULL) return Qnil;
 
   return Nokogiri_wrap_xml_node(rb_obj_class(self), dup);
@@ -344,7 +455,7 @@ static VALUE namespaced_key_eh(VALUE self, VALUE attribute, VALUE namespace)
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, node);
   if(xmlHasNsProp(node, (xmlChar *)StringValuePtr(attribute),
-        Qnil == namespace ? NULL : (xmlChar *)StringValuePtr(namespace)))
+        NIL_P(namespace) ? NULL : (xmlChar *)StringValuePtr(namespace)))
     return Qtrue;
   return Qfalse;
 }
@@ -379,13 +490,13 @@ static VALUE get(VALUE self, VALUE attribute)
   VALUE rval ;
   Data_Get_Struct(self, xmlNode, node);
 
-  if(attribute == Qnil) return Qnil;
+  if(NIL_P(attribute)) return Qnil;
 
   propstr = xmlGetProp(node, (xmlChar *)StringValuePtr(attribute));
 
-  if(NULL == propstr) return Qnil;
+  if(!propstr) return Qnil;
 
-  rval = NOKOGIRI_STR_NEW2(propstr, node->doc->encoding);
+  rval = NOKOGIRI_STR_NEW2(propstr);
 
   xmlFree(propstr);
   return rval ;
@@ -439,7 +550,7 @@ static VALUE attribute_with_ns(VALUE self, VALUE name, VALUE namespace)
   xmlAttrPtr prop;
   Data_Get_Struct(self, xmlNode, node);
   prop = xmlHasNsProp(node, (xmlChar *)StringValuePtr(name),
-      Qnil == namespace ? NULL : (xmlChar *)StringValuePtr(namespace));
+      NIL_P(namespace) ? NULL : (xmlChar *)StringValuePtr(namespace));
 
   if(! prop) return Qnil;
   return Nokogiri_wrap_xml_node(Qnil, (xmlNodePtr)prop);
@@ -454,12 +565,11 @@ static VALUE attribute_with_ns(VALUE self, VALUE name, VALUE namespace)
 static VALUE attribute_nodes(VALUE self)
 {
     /* this code in the mode of xmlHasProp() */
-    xmlNodePtr node ;
-    VALUE attr ;
+    xmlNodePtr node;
 
-    attr = rb_ary_new() ;
     Data_Get_Struct(self, xmlNode, node);
 
+    VALUE attr = rb_ary_new();
     Nokogiri_xml_node_properties(node, attr);
 
     return attr ;
@@ -520,7 +630,7 @@ static VALUE node_type(VALUE self)
 {
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, node);
-  return INT2NUM((int)node->type);
+  return INT2NUM((long)node->type);
 }
 
 /*
@@ -550,7 +660,7 @@ static VALUE get_content(VALUE self)
 
   xmlChar * content = xmlNodeGetContent(node);
   if(content) {
-    VALUE rval = NOKOGIRI_STR_NEW2(content, node->doc->encoding);
+    VALUE rval = NOKOGIRI_STR_NEW2(content);
     xmlFree(content);
     return rval;
   }
@@ -610,7 +720,7 @@ static VALUE get_name(VALUE self)
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, node);
   if(node->name)
-    return NOKOGIRI_STR_NEW2(node->name, node->doc->encoding);
+    return NOKOGIRI_STR_NEW2(node->name);
   return Qnil;
 }
 
@@ -627,7 +737,7 @@ static VALUE path(VALUE self)
   Data_Get_Struct(self, xmlNode, node);
   
   path = xmlGetNodePath(node);
-  VALUE rval = NOKOGIRI_STR_NEW2(path, node->doc->encoding);
+  VALUE rval = NOKOGIRI_STR_NEW2(path);
   xmlFree(path);
   return rval ;
 }
@@ -673,6 +783,8 @@ static VALUE native_write_to(
 
   xmlIndentTreeOutput = 1;
 
+  const char * before_indent = xmlTreeIndentString;
+
   xmlTreeIndentString = StringValuePtr(indent_string);
 
   xmlSaveCtxtPtr savectx = xmlSaveToIO(
@@ -680,11 +792,13 @@ static VALUE native_write_to(
       (xmlOutputCloseCallback)io_close_callback,
       (void *)io,
       RTEST(encoding) ? StringValuePtr(encoding) : NULL,
-      NUM2INT(options)
+      (int)NUM2INT(options)
   );
 
   xmlSaveTree(savectx, node);
   xmlSaveClose(savectx);
+
+  xmlTreeIndentString = before_indent;
   return io;
 }
 
@@ -699,7 +813,7 @@ static VALUE line(VALUE self)
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, node);
 
-  return INT2NUM(node->line);
+  return INT2NUM(xmlGetLineNo(node));
 }
 
 /*
@@ -717,10 +831,18 @@ static VALUE add_namespace_definition(VALUE self, VALUE prefix, VALUE href)
   xmlNsPtr ns = xmlNewNs(
       node,
       (const xmlChar *)StringValuePtr(href),
-      (const xmlChar *)(prefix == Qnil ? NULL : StringValuePtr(prefix))
+      (const xmlChar *)(NIL_P(prefix) ? NULL : StringValuePtr(prefix))
   );
 
-  if(Qnil == prefix) xmlSetNs(node, ns);
+  if(!ns) {
+    ns = xmlSearchNs(
+        node->doc,
+        node,
+        (const xmlChar *)(NIL_P(prefix) ? NULL : StringValuePtr(prefix))
+    );
+  }
+
+  if(NIL_P(prefix)) xmlSetNs(node, ns);
 
   return Nokogiri_wrap_xml_namespace(node->doc, ns);
 }
@@ -746,8 +868,11 @@ static VALUE new(int argc, VALUE *argv, VALUE klass)
   node->doc = doc->doc;
   NOKOGIRI_ROOT_NODE(node);
 
-  VALUE rb_node = Nokogiri_wrap_xml_node(klass, node);
-  rb_funcall2(rb_node, rb_intern("initialize"), argc, argv);
+  VALUE rb_node = Nokogiri_wrap_xml_node(
+      klass == cNokogiriXmlNode ? (VALUE)NULL : klass,
+      node
+  );
+  rb_obj_call_init(rb_node, argc, argv);
 
   if(rb_block_given_p()) rb_yield(rb_node);
 
@@ -766,12 +891,9 @@ static VALUE dump_html(VALUE self)
   xmlNodePtr node ;
   Data_Get_Struct(self, xmlNode, node);
 
-  if(node->doc->type == XML_DOCUMENT_NODE)
-    return rb_funcall(self, rb_intern("to_xml"), 0);
-
   buf = xmlBufferCreate() ;
   htmlNodeDump(buf, node->doc, node);
-  VALUE html = NOKOGIRI_STR_NEW2(buf->content, node->doc->encoding);
+  VALUE html = NOKOGIRI_STR_NEW2(buf->content);
   xmlBufferFree(buf);
   return html ;
 }
@@ -788,7 +910,7 @@ static VALUE compare(VALUE self, VALUE _other)
   Data_Get_Struct(self, xmlNode, node);
   Data_Get_Struct(_other, xmlNode, other);
 
-  return INT2NUM(xmlXPathCmpNodes(other, node));
+  return INT2NUM((long)xmlXPathCmpNodes(other, node));
 }
 
 VALUE Nokogiri_wrap_xml_node(VALUE klass, xmlNodePtr node)
@@ -831,13 +953,19 @@ VALUE Nokogiri_wrap_xml_node(VALUE klass, xmlNodePtr node)
       klass = cNokogiriXmlProcessingInstruction;
       break;
     case XML_ENTITY_DECL:
-      klass = cNokogiriXmlEntityDeclaration;
+      klass = cNokogiriXmlEntityDecl;
       break;
     case XML_CDATA_SECTION_NODE:
       klass = cNokogiriXmlCData;
       break;
     case XML_DTD_NODE:
-      klass = rb_const_get(mNokogiriXml, rb_intern("DTD"));
+      klass = cNokogiriXmlDtd;
+      break;
+    case XML_ATTRIBUTE_DECL:
+      klass = cNokogiriXmlAttributeDecl;
+      break;
+    case XML_ELEMENT_DECL:
+      klass = cNokogiriXmlElementDecl;
       break;
     default:
       klass = cNokogiriXmlNode;
@@ -848,13 +976,14 @@ VALUE Nokogiri_wrap_xml_node(VALUE klass, xmlNodePtr node)
   node->_private = (void *)rb_node;
 
   if (DOC_RUBY_OBJECT_TEST(node->doc) && DOC_RUBY_OBJECT(node->doc)) {
+    // it's OK if the document isn't fully realized (as in XML::Reader).
+    // see http://github.com/tenderlove/nokogiri/issues/closed/#issue/95
     document = DOC_RUBY_OBJECT(node->doc);
-    node_cache = rb_iv_get(document, "@node_cache");
+    node_cache = DOC_NODE_CACHE(node->doc);
+    rb_ary_push(node_cache, rb_node);
+    rb_funcall(document, decorate, 1, rb_node);
   }
 
-  rb_ary_push(node_cache, rb_node);
-  rb_funcall(document, rb_intern("decorate"), 1, rb_node);
-
   return rb_node ;
 }
 
@@ -871,7 +1000,6 @@ void Nokogiri_xml_node_properties(xmlNodePtr node, VALUE attr_list)
 
 VALUE cNokogiriXmlNode ;
 VALUE cNokogiriXmlElement ;
-VALUE cNokogiriXmlEntityDeclaration ;
 
 void init_xml_node()
 {
@@ -882,8 +1010,6 @@ void init_xml_node()
   cNokogiriXmlNode = klass;
 
   cNokogiriXmlElement = rb_define_class_under(xml, "Element", klass);
-  cNokogiriXmlEntityDeclaration =
-    rb_define_class_under(xml, "EntityDeclaration", klass);
 
   rb_define_singleton_method(klass, "new", new, -1);
 
@@ -915,6 +1041,9 @@ void init_xml_node()
   rb_define_method(klass, "dup", duplicate_node, -1);
   rb_define_method(klass, "unlink", unlink_node, 0);
   rb_define_method(klass, "internal_subset", internal_subset, 0);
+  rb_define_method(klass, "external_subset", external_subset, 0);
+  rb_define_method(klass, "create_internal_subset", create_internal_subset, 3);
+  rb_define_method(klass, "create_external_subset", create_external_subset, 3);
   rb_define_method(klass, "pointer_id", pointer_id, 0);
   rb_define_method(klass, "line", line, 0);
 
@@ -925,4 +1054,7 @@ void init_xml_node()
   rb_define_private_method(klass, "get", get, 1);
   rb_define_private_method(klass, "set_namespace", set_namespace, 1);
   rb_define_private_method(klass, "compare", compare, 1);
+
+  decorate = rb_intern("decorate");
+  decorate_bang = rb_intern("decorate!");
 }
