@@ -1,8 +1,13 @@
-
+# :stopdoc:
 module Nokogiri
-  module LibXML # :nodoc: all
+  module LibXML
     extend FFI::Library
-    ffi_lib 'xml2', 'xslt', 'exslt'
+    if RUBY_PLATFORM =~ /java/ && java.lang.System.getProperty('os.name') =~ /windows/i
+      raise(RuntimeError, "Nokogiri requires JRuby 1.4.0RC1 or later on Windows") if JRUBY_VERSION < "1.4.0RC1"
+      ffi_lib 'libxml2', 'libxslt', 'libexslt', 'msvcrt'
+    else
+      ffi_lib 'xml2', 'xslt', 'exslt'
+    end
 
     # globals.c
     attach_function :__xmlParserVersion, [], :pointer
@@ -28,7 +33,9 @@ else
 end
 
 module Nokogiri
-  module LibXML # :nodoc:
+  module LibXML
+    XML_CHAR_ENCODING_ERROR = -1
+
     # useful callback signatures
     callback :syntax_error_handler, [:pointer, :pointer], :void
     callback :generic_error_handler, [:pointer, :string], :void
@@ -50,10 +57,6 @@ module Nokogiri
     callback :start_element_ns_sax2_func, [:pointer, :pointer, :pointer, :pointer, :int, :pointer, :int, :int, :pointer], :void
     callback :end_element_ns_sax2_func, [:pointer, :pointer, :pointer, :pointer], :void
 
-    # libc
-    attach_function :calloc, [:int, :int], :pointer
-    attach_function :free, [:pointer], :void
-
     # HTMLparser.c
     attach_function :htmlReadMemory, [:string, :int, :string, :string, :int], :pointer
     attach_function :htmlReadIO, [:io_read_callback, :io_close_callback, :pointer, :string, :string, :int], :pointer
@@ -62,6 +65,9 @@ module Nokogiri
     attach_function :htmlEntityLookup, [:string], :pointer
     attach_function :htmlSAXParseFile, [:string, :pointer, :pointer, :pointer], :pointer # second arg 'encoding' should be a string, but we assign it as a pointer elsewhere
     attach_function :htmlSAXParseDoc, [:pointer, :pointer, :pointer, :pointer], :pointer # second arg 'encoding' should be a string, but we assign it as a pointer elsewhere
+    attach_function :htmlCreateMemoryParserCtxt, [:pointer, :int], :pointer
+    attach_function :htmlCreateFileParserCtxt, [:pointer, :pointer], :pointer
+    attach_function :htmlParseDocument, [:pointer], :int
 
     # HTMLtree.c
     attach_function :htmlDocDumpMemory, [:pointer, :pointer, :pointer], :void
@@ -74,6 +80,8 @@ module Nokogiri
     attach_function :xmlInitParser, [], :void
     attach_function :xmlReadIO, [:io_read_callback, :io_close_callback, :pointer, :string, :string, :int], :pointer
     attach_function :xmlCreateIOParserCtxt, [:pointer, :pointer, :io_read_callback, :io_close_callback, :pointer, :int], :pointer
+    attach_function :xmlCreateMemoryParserCtxt, [:pointer, :int], :pointer
+    attach_function :xmlCreateFileParserCtxt, [:pointer], :pointer
     attach_function :xmlSAXUserParseMemory, [:pointer, :pointer, :string, :int], :int
     attach_function :xmlSAXUserParseFile, [:pointer, :pointer, :string], :int
     attach_function :xmlParseDocument, [:pointer], :int
@@ -118,6 +126,7 @@ module Nokogiri
     attach_function :xmlStringGetNodeList, [:pointer, :pointer], :pointer # second arg should be a :string, but we only ship the results of xmlEncodeEntitiesReentrant, so let's optimize.
     attach_function :xmlNewNs, [:pointer, :string, :string], :pointer
     attach_function :xmlNewNsProp, [:pointer, :pointer, :string, :string], :pointer
+    attach_function :xmlSearchNs, [:pointer, :pointer, :string], :pointer
     attach_function :xmlSearchNsByHref, [:pointer, :pointer, :string], :pointer
     attach_function :xmlGetIntSubset, [:pointer], :pointer
     attach_function :xmlBufferCreate, [], :pointer
@@ -125,6 +134,13 @@ module Nokogiri
     attach_function :xmlSplitQName2, [:string, :pointer], :pointer # returns char* that must be freed
     attach_function :xmlNewDocProp, [:pointer, :string, :string], :pointer
     attach_function :xmlFreePropList, [:pointer], :void
+    attach_function :xmlCreateIntSubset, [:pointer] * 4, :pointer
+    attach_function :xmlNewDtd, [:pointer] * 4, :pointer
+
+    # valid.c
+    attach_function :xmlNewValidCtxt, [], :pointer
+    attach_function :xmlValidateDtd, [:pointer, :pointer, :pointer], :int
+    attach_function :xmlFreeValidCtxt, [:pointer], :void
 
     # xmlsave.c
     attach_function :xmlDocDumpMemory, [:pointer, :pointer, :pointer], :void
@@ -194,6 +210,7 @@ module Nokogiri
     attach_function :xmlResetLastError, [], :void
     attach_function :xmlCopyError, [:pointer, :pointer], :int
     attach_function :xmlGetLastError, [], :pointer
+    attach_function :xmlCtxtGetLastError, [:pointer], :pointer
 
     # hash.c
     attach_function :xmlHashScan, [:pointer, :hash_copier_callback, :pointer], :void
@@ -204,6 +221,8 @@ module Nokogiri
     attach_function :xmlTextReaderGetAttributeNo, [:pointer, :int], :pointer # returns char* that must be freed
     attach_function :xmlTextReaderLookupNamespace, [:pointer, :string], :pointer # returns char* that must be freed
     attach_function :xmlTextReaderRead, [:pointer], :int
+    attach_function :xmlTextReaderReadInnerXml, [:pointer], :pointer
+    attach_function :xmlTextReaderReadOuterXml, [:pointer], :pointer
     attach_function :xmlTextReaderAttributeCount, [:pointer], :int
     attach_function :xmlTextReaderCurrentNode, [:pointer], :pointer
     attach_function :xmlTextReaderExpand, [:pointer], :pointer
@@ -220,6 +239,7 @@ module Nokogiri
     attach_function :xmlTextReaderHasValue, [:pointer], :int
     attach_function :xmlFreeTextReader, [:pointer], :void
     attach_function :xmlReaderForIO, [:io_read_callback, :io_close_callback, :pointer, :string, :string, :int], :pointer
+    attach_function :xmlTextReaderNodeType, [:pointer], :int
 
     # xslt.c
     attach_function :xsltParseStylesheetDoc, [:pointer], :pointer
@@ -240,6 +260,7 @@ module Nokogiri
     attach_function :xmlSchemaSetParserStructuredErrors, [:pointer, :syntax_error_handler, :pointer], :void unless Nokogiri.is_2_6_16?
     attach_function :xmlSchemaParse, [:pointer], :pointer
     attach_function :xmlSchemaFreeParserCtxt, [:pointer], :void
+    attach_function :xmlSchemaNewDocParserCtxt, [:pointer], :pointer
 
     # relaxng.c
     attach_function :xmlRelaxNGNewValidCtxt, [:pointer], :pointer
@@ -250,6 +271,14 @@ module Nokogiri
     attach_function :xmlRelaxNGSetParserStructuredErrors, [:pointer, :syntax_error_handler, :pointer], :void unless Nokogiri.is_2_6_16?
     attach_function :xmlRelaxNGParse, [:pointer], :pointer
     attach_function :xmlRelaxNGFreeParserCtxt, [:pointer], :void
+    attach_function :xmlRelaxNGNewDocParserCtxt, [:pointer], :pointer
+
+    # libc
+    attach_function :calloc, [:int, :int], :pointer
+    attach_function :free, [:pointer], :void
+
+    attach_function :xmlParseCharEncoding, [:string], :int
+    attach_function :xmlSwitchEncoding, [:pointer, :int], :void
 
     # helpers
     def self.pointer_offset(n)
@@ -258,6 +287,8 @@ module Nokogiri
   end
 end
 
+# :startdoc:
+
 require 'nokogiri/syntax_error'
 require 'nokogiri/xml/syntax_error'
 
@@ -281,13 +312,23 @@ require 'nokogiri/xml/syntax_error'
   "structs/xml_text_reader",
   "structs/xml_sax_handler",
   "structs/xml_sax_push_parser_context",
+  "structs/xml_enumeration",
   "structs/html_elem_desc",
   "structs/html_entity_desc",
   "structs/xslt_stylesheet",
+  "structs/xml_parser_context",
+  "structs/xml_attribute",
+  "structs/xml_element",
+  "structs/xml_entity",
+  "structs/xml_element_content",
   "xml/node",
   "xml/namespace",
   "xml/dtd",
   "xml/attr",
+  "xml/attribute_decl",
+  "xml/element_decl",
+  "xml/element_content",
+  "xml/entity_decl",
   "xml/document",
   "xml/document_fragment",
   "xml/schema",
@@ -302,13 +343,14 @@ require 'nokogiri/xml/syntax_error'
   "xml/syntax_error",
   "xml/reader",
   "xml/entity_reference",
+  "xml/sax/parser_context",
   "xml/sax/parser",
   "xml/sax/push_parser",
   "html/document",
   "html/element_description",
   "html/entity_lookup",
-  "html/sax/parser",
+  "html/sax/parser_context",
   "xslt/stylesheet",
 ].each do |file|
-  require File.join(File.dirname(__FILE__), file)
+  require "nokogiri/ffi/#{file}"
 end
