@@ -1,16 +1,42 @@
+# :stopdoc:
 module Nokogiri
   module XML
     class DTD < Node
-      
-      def elements # :nodoc:
+      def validate document
+        error_list = []
+        ctxt = LibXML.xmlNewValidCtxt
+
+        LibXML.xmlSetStructuredErrorFunc(nil, SyntaxError.error_array_pusher(error_list))
+        LibXML.xmlValidateDtd ctxt, document.cstruct, cstruct
+
+        LibXML.xmlSetStructuredErrorFunc nil, nil
+
+        LibXML.xmlFreeValidCtxt ctxt
+
+        error_list
+      end
+
+      def system_id
+        cstruct[:system_id]
+      end
+
+      def external_id
+        cstruct[:external_id]
+      end
+
+      def elements
         internal_attributes :elements
       end
 
-      def entities # :nodoc:
+      def entities
         internal_attributes :entities
       end
 
-      def notations # :nodoc:
+      def attributes
+        internal_attributes :attributes
+      end
+
+      def notations
         attr_ptr = cstruct[:notations]
         return nil if attr_ptr.null?
 
@@ -24,8 +50,9 @@ module Nokogiri
         ahash
       end
 
-    private
-      def internal_attributes(attr_name) # :nodoc:
+      private
+
+      def internal_attributes attr_name
         attr_ptr = cstruct[attr_name.to_sym]
         return nil if attr_ptr.null?
 
@@ -36,7 +63,7 @@ module Nokogiri
         LibXML.xmlHashScan(attr_ptr, proc, nil)
         ahash
       end
-
     end
   end
 end
+# :startdoc:
