@@ -1,6 +1,7 @@
 require 'nokogiri/html/entity_lookup'
 require 'nokogiri/html/document'
 require 'nokogiri/html/document_fragment'
+require 'nokogiri/html/sax/parser_context'
 require 'nokogiri/html/sax/parser'
 require 'nokogiri/html/element_description'
 
@@ -8,7 +9,7 @@ module Nokogiri
   class << self
     ###
     # Parse HTML.  Convenience method for Nokogiri::HTML::Document.parse
-    def HTML thing, url = nil, encoding = nil, options = 2145, &block
+    def HTML thing, url = nil, encoding = nil, options = XML::ParseOptions::DEFAULT_HTML, &block
       Nokogiri::HTML::Document.parse(thing, url, encoding, options, &block)
     end
   end
@@ -17,7 +18,7 @@ module Nokogiri
     class << self
       ###
       # Parse HTML.  Convenience method for Nokogiri::HTML::Document.parse
-      def parse thing, url = nil, encoding = nil, options = 2145, &block
+      def parse thing, url = nil, encoding = nil, options = XML::ParseOptions::DEFAULT_HTML, &block
         Document.parse(thing, url, encoding, options, &block)
       end
 
