@@ -30,7 +30,8 @@ module Nokogiri
         # Parse html stored in +data+ using +encoding+
         def parse_memory data, encoding = 'UTF-8'
           raise ArgumentError unless data
-          native_parse_memory(data, encoding)
+          return unless data.length > 0
+          ParserContext.memory(data, encoding).parse_with self
         end
 
         ###
@@ -39,7 +40,7 @@ module Nokogiri
           raise ArgumentError unless filename
           raise Errno::ENOENT unless File.exists?(filename)
           raise Errno::EISDIR if File.directory?(filename)
-          native_parse_file filename, encoding
+          ParserContext.file(filename, encoding).parse_with self
         end
       end
     end
