@@ -34,6 +34,8 @@ module Nokogiri
     #
     # You may search this node's subtree using Node#xpath and Node#css
     class Node
+      include Nokogiri::XML::PP::Node
+
       # Element node type, see Nokogiri::XML::Node#element?
       ELEMENT_NODE =       1
       # Attribute node type
@@ -198,6 +200,24 @@ module Nokogiri
       end
       alias :% :at
 
+      ##
+      # Search this node for the first occurrence of XPath +paths+.
+      # Equivalent to <tt>xpath(paths).first</tt>
+      # See Node#xpath for more information.
+      #
+      def at_xpath *paths
+        xpath(*paths).first
+      end
+
+      ##
+      # Search this node for the first occurrence of CSS +rules+.
+      # Equivalent to <tt>css(rules).first</tt>
+      # See Node#css for more information.
+      #
+      def at_css *rules
+        css(*rules).first
+      end
+
       ###
       # Get the attribute value for the attribute +name+
       def [] name
@@ -295,7 +315,7 @@ module Nokogiri
       def inner_html= tags
         children.each { |x| x.remove}
 
-        fragment(tags).children.to_a.reverse.each do |node|
+        fragment(tags).children.to_a.each do |node|
           add_child node
         end
         self
@@ -388,8 +408,8 @@ module Nokogiri
       end
 
       # Get the inner_html for this node's Node#children
-      def inner_html
-        children.map { |x| x.to_html }.join
+      def inner_html *args
+        children.map { |x| x.to_html(*args) }.join
       end
 
       # Get the path to this node as a CSS expression
@@ -399,14 +419,6 @@ module Nokogiri
         }.compact.join(' > ')
       end
 
-      #  recursively get all namespaces from this node and its subtree
-      def collect_namespaces
-        # TODO: print warning message if a prefix refers to more than one URI in the document?
-        ns = {}
-        traverse {|j| ns.merge!(j.namespaces)}
-        ns
-      end
-
       ###
       # Get a list of ancestor Node for this Node.  If +selector+ is given,
       # the ancestors must match +selector+
@@ -461,8 +473,9 @@ module Nokogiri
       end
 
       ####
-      #  replace this Node with the +new_node+ in the Document.
-      def replace(new_node)
+      # +replace+ this Node with the +new_node+ in the Document.  The new node
+      # must be a Nokogiri::XML::Node or a non-empty String
+      def replace new_node
         if new_node.is_a?(Document) || !new_node.is_a?(XML::Node)
           raise ArgumentError, <<-EOERR
 Node.replace requires a Node argument, and cannot accept a Document.
@@ -481,8 +494,8 @@ Node.replace requires a Node argument, and cannot accept a Document.
       end
 
       ###
-      # Serialize Node using +options+.  Save options 
-      # can also be set using a block. See SaveOptions.
+      # Serialize Node using +options+.  Save options can also be set using a
+      # block. See SaveOptions.
       #
       # These two statements are equivalent:
       #
@@ -495,27 +508,20 @@ Node.replace requires a Node argument, and cannot accept a Document.
       #   end
       #
       def serialize *args, &block
-        if args.first && !args.first.is_a?(Hash)
-          $stderr.puts(<<-eowarn)
-#{self.class}#serialize(encoding, save_opts) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called serialize from here:
-
-  #{caller.first}
-
-Please change to #{self.class}#serialize(:encoding => enc, :save_with => opts)
-          eowarn
-        end
-
         options = args.first.is_a?(Hash) ? args.shift : {
           :encoding   => args[0],
           :save_with  => args[1] || SaveOptions::FORMAT
         }
 
-        io = StringIO.new
+        encoding = options[:encoding] || document.encoding
+
+        outstring = ""
+        if encoding && outstring.respond_to?(:force_encoding)
+          outstring.force_encoding(Encoding.find(encoding))
+        end
+        io = StringIO.new(outstring)
         write_to io, options, &block
-        io.rewind
-        io.read
+        io.string
       end
 
       ###
@@ -526,19 +532,6 @@ Please change to #{self.class}#serialize(:encoding => enc, :save_with => opts)
       # See Node#write_to for a list of +options+.  For formatted output,
       # use Node#to_xhtml instead.
       def to_html options = {}
-        if options.is_a?(String)
-          $stderr.puts(<<-eowarn)
-Node#to_html(encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called to_html from here:
-
-  #{caller.first}
-
-Please change to Node#to_html(:encoding => #{options})
-          eowarn
-          options = { :encoding => options }
-        end
-
         # FIXME: this is a hack around broken libxml versions
         return dump_html if %w[2 6] === LIBXML_VERSION.split('.')[0..1]
 
@@ -559,21 +552,6 @@ Please change to Node#to_html(:encoding => #{options})
       def to_xml options = {}
         encoding = nil
 
-        # FIXME add a deprecation warning
-        if options.is_a? String
-          $stderr.puts(<<-eowarn)
-Node#to_xml(encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called to_xml from here:
-
-  #{caller.first}
-
-Please change to Node#to_xml(:encoding => #{options})
-          eowarn
-          options = {
-            :encoding => options
-          }
-        end
         options[:save_with] ||= SaveOptions::FORMAT | SaveOptions::AS_XML
 
         serialize(options)
@@ -586,19 +564,6 @@ Please change to Node#to_xml(:encoding => #{options})
       #
       # See Node#write_to for a list of +options+
       def to_xhtml options = {}
-        if options.is_a?(String)
-          options = { :encoding => options }
-          $stderr.puts(<<-eowarn)
-Node#to_xml(encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called to_xhtml from here:
-
-  #{caller.first}
-
-Please change to Node#to_xhtml(:encoding => #{options})
-          eowarn
-        end
-
         # FIXME: this is a hack around broken libxml versions
         return dump_html if %w[2 6] === LIBXML_VERSION.split('.')[0..1]
 
@@ -628,18 +593,6 @@ Please change to Node#to_xhtml(:encoding => #{options})
       #   node.write_to(io, :indent_text => '-', :indent => 2
       #
       def write_to io, *options
-        if options.length > 0 && !options.first.is_a?(Hash)
-          $stderr.puts(<<-eowarn)
-Node#write_to(io, encoding, save_options) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called write_to from here:
-
-  #{caller.first}
-
-Please change to: Node#write_to(io, :encoding => e, :save_options => opts)
-          eowarn
-        end
-
         options       = options.first.is_a?(Hash) ? options.shift : {}
         encoding      = options[:encoding] || options[0]
         save_options  = options[:save_with] || options[1] || SaveOptions::FORMAT
@@ -658,19 +611,6 @@ Please change to: Node#write_to(io, :encoding => e, :save_options => opts)
       #
       # See Node#write_to for a list of +options+
       def write_html_to io, options = {}
-        if options.is_a?(String)
-          $stderr.puts(<<-eowarn)
-Node#write_html_to(io, encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called write_html_to from here:
-
-  #{caller.first}
-
-Please change to Node#write_html_to(io, :encoding => #{options})
-          eowarn
-          options = { :encoding => options }
-        end
-
         # FIXME: this is a hack around broken libxml versions
         return (io << dump_html) if %w[2 6] === LIBXML_VERSION.split('.')[0..1]
 
@@ -686,19 +626,6 @@ Please change to Node#write_html_to(io, :encoding => #{options})
       #
       # See Node#write_to for a list of +options+
       def write_xhtml_to io, options = {}
-        if options.is_a?(String)
-          $stderr.puts(<<-eowarn)
-Node#write_xhtml_to(io, encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called write_xhtml_to from here:
-
-  #{caller.first}
-
-Please change to Node#write_xhtml_to(io, :encoding => #{options})
-          eowarn
-          options = { :encoding => options }
-        end
-
         # FIXME: this is a hack around broken libxml versions
         return (io << dump_html) if %w[2 6] === LIBXML_VERSION.split('.')[0..1]
 
@@ -716,18 +643,6 @@ Please change to Node#write_xhtml_to(io, :encoding => #{options})
       #
       # See Node#write_to for a list of options
       def write_xml_to io, options = {}
-        if options.is_a?(String)
-          $stderr.puts(<<-eowarn)
-Node#write_xml_to(io, encoding) is deprecated and will be removed in
-Nokogiri version 1.4.0 *or* after June 1 2009.
-You called write_xml_to from here:
-
-  #{caller.first}
-
-Please change to Node#write_xml_to(io, :encoding => #{options})
-          eowarn
-          options = { :encoding => options }
-        end
         options[:save_with] ||= SaveOptions::FORMAT | SaveOptions::AS_XML
         write_to io, options
       end
@@ -740,8 +655,11 @@ Please change to Node#write_xml_to(io, :encoding => #{options})
         return nil unless document == other.document
         compare other
       end
+
+      private
+      def inspect_attributes
+        [:name, :namespace, :attribute_nodes, :children]
+      end
     end
   end
 end
-
-class Nokogiri::XML::Element < Nokogiri::XML::Node ; end
