@@ -1,4 +1,4 @@
-require File.expand_path(File.join(File.dirname(__FILE__), '..', "helper"))
+require "helper"
 
 require 'stringio'
 
@@ -10,6 +10,44 @@ module Nokogiri
         @xml = Nokogiri::XML(File.read(XML_FILE), XML_FILE)
       end
 
+      def test_element?
+        assert @xml.root.element?, 'is an element'
+      end
+
+      def test_slash_search
+        assert_equal 'EMP0001', (@xml/:staff/:employee/:employeeId).first.text
+      end
+
+      def test_append_with_document
+        assert_raises(ArgumentError) do
+          @xml.root << Nokogiri::XML::Document.new
+        end
+      end
+
+      def test_inspect_ns
+        xml = Nokogiri::XML(<<-eoxml) { |c| c.noblanks }
+          <root xmlns="http://tenderlovemaking.com/" xmlns:foo="bar">
+            <awesome/>
+          </root>
+        eoxml
+        ins = xml.inspect
+
+        xml.traverse do |node|
+          assert_match node.class.name, ins
+          if node.respond_to? :attributes
+            node.attributes.each do |k,v|
+              assert_match k, ins
+              assert_match v, ins
+            end
+          end
+
+          if node.respond_to?(:namespace) && node.namespace
+            assert_match node.namespace.class.name, ins
+            assert_match node.namespace.href, ins
+          end
+        end
+      end
+
       def test_namespace_nodes
         xml = Nokogiri::XML <<-eoxml
           <root xmlns="http://tenderlovemaking.com/" xmlns:foo="bar">
@@ -114,13 +152,21 @@ module Nokogiri
         end
       end
 
+      def test_fragment_creates_elements
+        apple = @xml.fragment('<Apple/>')
+        apple.children.each do |child|
+          assert_equal Nokogiri::XML::Node::ELEMENT_NODE, child.type
+          assert_instance_of Nokogiri::XML::Element, child
+        end
+      end
+
       def test_node_added_to_root_should_get_namespace
         fruits = Nokogiri::XML(<<-eoxml)
           <Fruit xmlns='http://www.fruits.org'>
           </Fruit>
         eoxml
         apple = fruits.fragment('<Apple/>')
-        fruits << apple
+        fruits.root << apple
         assert_equal 1, fruits.xpath('//xmlns:Apple').length
       end
 
@@ -157,6 +203,13 @@ module Nokogiri
         assert_equal 'http://tenderlovemaking.com', node.namespaces['xmlns:foo']
       end
 
+      def test_add_namespace_twice
+        node = @xml.at('address')
+        ns = node.add_namespace('foo', 'http://tenderlovemaking.com')
+        ns2 = node.add_namespace('foo', 'http://tenderlovemaking.com')
+        assert_equal ns, ns2
+      end
+
       def test_add_default_ns
         node = @xml.at('address')
         node.add_namespace(nil, 'http://tenderlovemaking.com')
@@ -229,6 +282,20 @@ module Nokogiri
         assert_equal node, @xml.xpath('//address').first
       end
 
+      def test_at_xpath
+        node = @xml.at_xpath('//address')
+        nodes = @xml.xpath('//address')
+        assert_equal 5, nodes.size
+        assert_equal node, nodes.first
+      end
+
+      def test_at_css
+        node = @xml.at_css('address')
+        nodes = @xml.css('address')
+        assert_equal 5, nodes.size
+        assert_equal node, nodes.first
+      end
+
       def test_percent
         node = @xml % ('address')
         assert_equal node, @xml.xpath('//address').first
@@ -278,6 +345,23 @@ module Nokogiri
         assert doc.at('//xmlns:second')
       end
 
+      def test_add_child_should_not_inherit_namespace_if_it_has_one
+        doc = Nokogiri::XML(<<-eoxml)
+          <root xmlns="http://tenderlovemaking.com/" xmlns:foo="http://flavorjon.es/">
+            <first>
+            </first>
+          </root>
+        eoxml
+        assert node = doc.at('//xmlns:first')
+        child = Nokogiri::XML::Node.new('second', doc)
+
+        ns = doc.root.namespace_definitions.detect { |x| x.prefix == "foo" }
+        child.namespace = ns
+
+        node.add_child(child)
+        assert doc.at('//foo:second', "foo" => "http://flavorjon.es/")
+      end
+
       def test_write_to
         io = StringIO.new
         @xml.write_to io
@@ -357,7 +441,7 @@ module Nokogiri
       def test_new
         assert node = Nokogiri::XML::Node.new('input', @xml)
         assert_equal 1, node.node_type
-        assert_instance_of Nokogiri::XML::Node, node
+        assert_instance_of Nokogiri::XML::Element, node
       end
 
       def test_to_str
@@ -722,12 +806,9 @@ EOF
         assert_equal 'http://foo.com/', namespaces['xmlns:a']
         assert namespaces.key?('xmlns:b')
         assert_equal 'http://bar.com/', namespaces['xmlns:b']
-        assert namespaces.key?('xmlns:c')
-        assert_equal 'http://bazz.com/', namespaces['xmlns:c']
 
         assert_equal "hello a", xml.search("//a:div", xml.namespaces).first.inner_text
         assert_equal "hello b", xml.search("//b:div", xml.namespaces).first.inner_text
-        assert_equal "hello c", xml.search("//c:div", xml.namespaces).first.inner_text
       end
 
       def test_namespace
